#global variables and functions for graphic display management
#to be imported with
#import graphicDisplayGlobalVarAndFunctions as gvf

#useful links

#labels and colors in networkX
#https://networkx.github.io/documentation/latest/examples/drawing/labels_and_colors.html

#Matplotlib colors
#http://matplotlib.org/api/colors_api.html

#html colors
#http://www.w3schools.com/html/html_colornames.asp


import networkx as nx
import matplotlib.pyplot as plt
import matplotlib as mplt
import commonVar as common


# the base: creating the graph (and copying its address in a common variable
# to have the possibility of direct interaction with the graph when
# the program is finished, as the common space is imported also in the main
# program
def createGraph():
    global agentGraph, colors, pos, labels, edge_labels

    print "\nMatplotlib version %s running\n" % mplt.__version__
    
    agentGraph=nx.DiGraph() # directed graph, instead of nx.Graph()
    colors={}
    pos={}
    labels={}
    edge_labels={}
    common.g=agentGraph               #copy the address of the graph
    common.g_labels=labels            #copy the address of the labels
    common.g_edge_labels=edge_labels  #copy the address of the labels of the edges


# searching tools

def findANodeFromLabel(label):
    for i in range(len(labels)):
        if labels.items()[i][1][:4]==label: #[:4] to conpare only the 4 initial char.
            return labels.items()[i][0]
    # if not found implicitly returns None

def findNodesFromSector(sector):
    nodeList=[]
    for aNode in agentGraph.nodes():
        if agentGraph.node[aNode]['sector']==sector:
            nodeList.append(aNode)
    return nodeList

def createEdge(a, b):
    # implicitly directed, due to the use of DiGraph
    if a == None or b == None:
       print "Internal error, attempt to create an edge with a node define None"
       exit(0)


    try:
      agentGraph[a][b]['weight']= 1+agentGraph[a][b]['weight']
    except:
      agentGraph.add_edge(a,b)
      agentGraph[a][b]['weight']=1

    if a != b:
     # verifying the presence of the edge in the other direction
     try:
      otherW = agentGraph[b][a]['weight']
      edge_labels[a,b]="w.s %d and %d" % (agentGraph[a][b]['weight'],otherW)	
      edge_labels[b,a]=""
     except:
      edge_labels[a,b]="w. %d" % agentGraph[a][b]['weight']

    if a == b:
      edge_labels[a,b]=""
      agentGraph[a][b]['pseudoLabel']="auto link w. %d" \
                                      % agentGraph[a][b]['weight']



# using networkX and matplotlib case 
def closeNetworkXdisplay(): 
    plt.close()

def openClearNetworkXdisplay():
    plt.ion()
    #plt.clf()

def clearNetworkXdisplay():
    plt.clf()

def getGraph():
    try:
        return agentGraph
    except:
        return 0

def pruneEdges():

    if not common.prune: return
    common.prune=False
    
    print "New threshold to prune: < %d" % common.pruneThreshold

    edges=agentGraph.edges()
    print "weights of the links"
    for anEdge in edges:
        u = labels[anEdge[0]][0:4]
        uu= anEdge[0]
        v = labels[anEdge[1]][0:4]
        vv= anEdge[1]
        w = agentGraph[anEdge[0]][anEdge[1]]["weight"]
        print u, v, w

        if w < common.pruneThreshold:
            # managing labels, related to createEdge above
            edge_labels[uu,vv]=""
            try: edge_labels[vv,uu]="w. %d" % agentGraph[vv][uu]['weight']
            except: pass
            if uu==vv: agentGraph[uu][uu]['pseudoLabel']=""
            labels[uu]=str(uu.number)+" ("+str(len(uu.recipeWaitingList))+")"
            #removing
            agentGraph.remove_edge(uu, vv)


def drawGraph():


    #directed, due to the use of DiGraph

    # draw_netwokx is well documented at
    # https://networkx.github.io/documentation/latest/reference/
    # generated/networkx.drawing.nx_pylab.draw_networkx.html 
    #nx.draw_networkx(agentGraph,    font_size=10,node_size=500, \
    clearNetworkXdisplay()
    pruneEdges()
    nx.draw_networkx(agentGraph,pos,font_size=10,node_size=500, \
         node_color=colors.values(), \
         labels = labels)
    nx.draw_networkx_edge_labels(agentGraph,pos,edge_labels=edge_labels,\
                                 font_size=9)
    plt.draw()
    #plt.show()

    #print agentGraph.nodes(data=True)
    #print agentGraph.edges(data=True)
    #print labels
    #print edge_labels

    #print a, agentGraph.node[a].keys(), agentGraph.node[a].values(),\
    #      agentGraph.node[a]['sector']


    # adjacency
    print
    for i in range(len(common.orderedListOfNodes)):
      print "%d " % common.orderedListOfNodes[i].number,
    print
    A = nx.adjacency_matrix(agentGraph, nodelist=common.orderedListOfNodes, \
                              weight='weight')
    #print A          # as sparse matrix, defaul from nx 1.9.1
    print A.todense() # as a regular matrix
    print

    # neighbors

    for aNode in agentGraph.nodes():
        print labels[aNode][0:4], [labels[node][0:4] \
                                   for node in nx.neighbors(agentGraph,aNode)]

    # betweenness_centrality
    # Betweenness centrality of a node v is the sum of the fraction of all-pairs
    # shortest paths that pass through v
    # http://networkx.lanl.gov/reference/generated/
    # networkx.algorithms.centrality.betweenness_centrality.html
    print
    print "betweenness_centrality" 
    btwn = nx.betweenness_centrality(agentGraph, normalized=True, weight='weight')
    #print btw
    for i in range(len(common.orderedListOfNodes)):
        print common.orderedListOfNodes[i].number, btwn[common.orderedListOfNodes[i]]

    # closeness_centrality
    # Closeness centrality at a node is 1/average distance to all other nodes
    # http://networkx.lanl.gov/reference/generated/
    # networkx.algorithms.centrality.closeness_centrality.html
    print
    print "closeness_centrality"
    clsn = nx.closeness_centrality(agentGraph, normalized=True)
    #print clsn
    for i in range(len(common.orderedListOfNodes)):
        print common.orderedListOfNodes[i].number, clsn[common.orderedListOfNodes[i]]
    
